/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Authors: Christian Hitz, digitalSTROM AG <christian.hitz@digitalstrom.com>
             Michael Tross, digitalSTROM AG <michael.tross@digitalstrom.com>
             Reto Flütsch, digitalSTROM AG <reto.fluetsch@digitalstrom.com>

    This file is part of vdSM.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.
    

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vdSM. If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
    #include "config.h"
#endif

#include "selectA-socket.h"
#include "device-api-proto.h"

#include "util.h"
#include "channelutil.h"
#include "datamodel.h"

#include <syslog.h>
#include <utlist.h>
#include <stdlib.h>

void SelectASocket_SendSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene, bool force)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d %scall scene %d\n", localZoneId, groupId, force? "force ":"", scene);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_scene(dsuid->dsuid, scene, force);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendSceneDevice(uint16_t deviceId, Scene scene, bool force)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_send_scene(dsuid, scene, force);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device send scene. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendUndoSceneNumberZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d call undo scene number %d\n", localZoneId, groupId, scene);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_undo_scene_number(dsuid->dsuid, scene);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendUndoSceneZoneGroup(uint8_t localZoneId, uint8_t groupId)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d call undo scene\n", localZoneId, groupId);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_undo_scene(dsuid->dsuid);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendUndoSceneNumberDevice(uint16_t deviceId, Scene scene)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_send_undo_scene_number(dsuid, scene);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device send undo scene number. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendUndoSceneDevice(uint16_t deviceId)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_send_undo_scene(dsuid);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device send undo scene. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendBlinkZoneGroup(uint8_t localZoneId, uint8_t groupId)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d blink\n", localZoneId, groupId);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_identify(dsuid->dsuid);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendBlinkDevice(uint16_t deviceId)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_identify(dsuid);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device identify. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendSetSceneLocalPrioZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d set local priority\n", localZoneId, groupId);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_local_prio(dsuid->dsuid, scene);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_CallSceneMinZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d call min scene %d \n", localZoneId, groupId, scene);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_min_scene(dsuid->dsuid, scene);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendProgOnZoneGroup(uint16_t zoneId __attribute__((unused)),
                                       uint8_t groupId __attribute__((unused)))
{
    // TODO: implement
}

void SelectASocket_SendProgOffZoneGroup(uint16_t zoneId __attribute__((unused)),
                                        uint8_t groupId __attribute__((unused)))
{
    // TODO: implement
}

void SelectASocket_SendProgOnDevice(uint16_t deviceId __attribute__((unused)))
{
    // TODO: implement
}

void SelectASocket_SendProgOffDevice(uint16_t deviceId __attribute__((unused)))
{
    // TODO: implement
}

void SelectASocket_SendSaveSceneZoneGroup(uint8_t localZoneId, uint8_t groupId, Scene scene)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d save scene %d\n", localZoneId, groupId, scene);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_save_scene(dsuid->dsuid, scene);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendOPCDimDevice(uint16_t deviceId, OPCOperation operation, uint8_t channelId)
{
    char dsuid[DSUID_STR_LEN];
    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_send_dim_channel(dsuid, channelId, operation, /*area*/ 0);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in OPC Dim Device. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendOPCDimZoneGroup(uint8_t zoneId, uint8_t groupId, OPCOperation operation, uint8_t channelId)
{
    vdsm_report(LOG_DEBUG, "selectA-Socket. in opc dim zone %d and group %d operation %d channel %d\n", zoneId, groupId, operation, channelId);
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;
    Datamodel_ZoneGroupGetDeviceDsuids(zoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        device_api_send_dim_channel(dsuid->dsuid, channelId, operation, /*area*/ 0);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendSaveSceneDevice(uint16_t deviceId, Scene scene)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_send_save_scene(dsuid, scene);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device save scene. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendSetOutvalZoneGroup(uint8_t localZoneId, uint8_t groupId, uint8_t outval)
{
    device_dsuid_t *dsuid_list = NULL, *dsuid, *tmp_dsuid;

    vdsm_report(LOG_DEBUG, "selectA-Socket. in local-zone %d and group %d set outval %d\n", localZoneId, groupId, outval);
    Datamodel_ZoneGroupGetDeviceDsuids(localZoneId, groupId, &dsuid_list);

    LL_FOREACH_SAFE(dsuid_list, dsuid, tmp_dsuid) {
        uint8_t channel = 0;
        device_api_set_outval(dsuid->dsuid, channel, outval);
        LL_DELETE(dsuid_list, dsuid);
        free(dsuid);
    }
}

void SelectASocket_SendSetOutvalDevice(uint16_t deviceId, uint8_t outval)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        uint8_t channel = 0;
        device_api_set_outval(dsuid, channel, outval);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device set outval. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendAddGroupDevice(uint16_t deviceId, uint8_t groupId)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_add_group(dsuid, groupId);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device add group. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendRemoveGroupDevice(uint16_t deviceId, uint8_t groupId)
{
    char dsuid[DSUID_STR_LEN];

    if (Datamodel_GetDsuidFromDeviceId(deviceId, dsuid)) {
        device_api_remove_group(dsuid, groupId);
    } else {
        vdsm_report(LOG_WARNING, "selectA-Socket in device remove group. device: %d not found\n", deviceId);
    }
}

void SelectASocket_SendRemoveGroupFromZone(uint8_t localZoneId __attribute__((unused)),
                                           uint8_t groupId __attribute__((unused)))
{
    vdsm_report(LOG_DEBUG, "selectA-Socket in remove group from zone. function is not implemented (handled in remove device)\n");
}

